import Dispatcher from './dispatcher'

export declare class RedirectHandler implements Dispatcher.DispatchHandler {
  constructor (
    dispatch: Dispatcher.Dispatch,
    maxRedirections: number,
    opts: Dispatcher.DispatchOptions,
    handler: Dispatcher.DispatchHandler,
    redirectionLimitReached: boolean
  )
}

export declare class DecoratorHandler implements Dispatcher.DispatchHandler {
  constructor (handler: Dispatcher.DispatchHandler)
}
